#!/usr/bin/env python3
"""
Copyright 2022 Byron Labs S.L.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import IntEnum


class Services(IntEnum):

    acr_nema = 104
    afpovertcp = 548
    afs3_bos = 7007
    afs3_callback = 7001
    afs3_fileserver = 7000
    afs3_kaserver = 7004
    afs3_prserver = 7002
    afs3_rmtsys = 7009
    afs3_update = 7008
    afs3_vlserver = 7003
    afs3_volser = 7005
    amanda = 10080
    amandaidx = 10082
    amidxtape = 10083
    amqp = 5672
    amqps = 5671
    asf_rmcp = 623
    asp = 27374
    auth = 113
    babel = 6696
    bacula_dir = 9101
    bacula_fd = 9102
    bacula_sd = 9103
    bbs = 7000
    bgp = 179
    bgpd = 2605
    biff = 512
    binkp = 24554
    bootpc = 68
    bootps = 67
    canna = 5680
    cfengine = 5308
    chargen = 19
    cisco_sccp = 2000
    clc_build_daemon = 8990
    clearcase = 371
    cmip_agent = 164
    cmip_man = 163
    codaauth2 = 370
    codasrv = 2432
    codasrv_se = 2433
    csync2 = 30865
    cvspserver = 2401
    daap = 3689
    datametrics = 1645
    daytime = 13
    db_lsp = 17500
    dcap = 22125
    dhcpv6_client = 546
    dhcpv6_server = 547
    dicom = 11112
    dict = 2628
    dircproxy = 57000
    discard = 9
    distcc = 3632
    domain = 53
    domain_s = 853
    echo = 7
    epmap = 135
    epmd = 4369
    exec = 512
    f5_globalsite = 2792
    f5_iquery = 4353
    fax = 4557
    fido = 60179
    finger = 79
    font_service = 7100
    freeciv = 5556
    fsp = 21
    ftp = 21
    ftp_data = 20
    ftps = 990
    ftps_data = 989
    gdomap = 538
    gds_db = 3050
    git = 9418
    gnunet = 2086
    gnutella_rtr = 6347
    gnutella_svc = 6346
    gopher = 70
    gpsd = 2947
    gris = 2135
    groupwise = 1677
    gsidcap = 22128
    gsiftp = 2811
    gsigatekeeper = 2119
    hkp = 11371
    http = 80
    http_alt = 8080
    https = 443
    hylafax = 4559
    iax = 4569
    icpv2 = 3130
    imap2 = 143
    imaps = 993
    ingreslock = 1524
    ipp = 631
    iprop = 2121
    ipsec_nat_t = 4500
    ipx = 213
    ircd = 6667
    ircs_u = 6697
    isakmp = 500
    iscsi_target = 3260
    isisd = 2608
    isns = 3205
    iso_tsap = 102
    kamanda = 10081
    kerberos = 88
    kerberos4 = 750
    kerberos_adm = 749
    kerberos_master = 751
    kermit = 1649
    klogin = 543
    kpasswd = 464
    krb_prop = 754
    kshell = 544
    l2f = 1701
    ldap = 389
    ldaps = 636
    ldp = 646
    login = 513
    lotusnote = 1352
    mailq = 174
    mdns = 5353
    microsoft_ds = 445
    moira_db = 775
    moira_update = 777
    moira_ureg = 779
    mon = 2583
    ms_sql_m = 1434
    ms_sql_s = 1433
    ms_wbt_server = 3389
    mtn = 4691
    munin = 4949
    mysql = 3306
    mysql_proxy = 6446
    nbd = 10809
    nbp = 2
    netbios_dgm = 138
    netbios_ns = 137
    netbios_ssn = 139
    netstat = 15
    nfs = 2049
    nntp = 119
    nntps = 563
    nqs = 607
    nrpe = 5666
    nsca = 5667
    ntalk = 518
    ntp = 123
    ntske = 4460
    nut = 3493
    omniorb = 8088
    openvpn = 1194
    ospf6d = 2606
    ospfapi = 2607
    ospfd = 2604
    passwd_server = 752
    pawserv = 345
    pop3 = 110
    pop3s = 995
    poppassd = 106
    postgresql = 5432
    predict = 1210
    printer = 515
    proofd = 1093
    ptp_event = 319
    ptp_general = 320
    puppet = 8140
    qmqp = 628
    qmtp = 209
    qotd = 17
    radius = 1812
    radius_acct = 1813
    radmin_port = 4899
    redis = 6379
    remctl = 4373
    ripd = 2602
    ripngd = 2603
    rmiregistry = 1099
    rmtcfg = 1236
    rootd = 1094
    route = 520
    rpc2portmap = 369
    rplay = 5555
    rsync = 873
    rtcm_sc104 = 2101
    rtmp = 1
    rtsp = 554
    saft = 487
    sa_msg_port = 1646
    sane_port = 6566
    sge_execd = 6445
    sge_qmaster = 6444
    sgi_cad = 17004
    sgi_cmsd = 17001
    sgi_crsd = 17002
    sgi_gcd = 17003
    shell = 514
    sieve = 4190
    silc = 706
    sip = 5060
    sip_tls = 5061
    skkserv = 1178
    smtp = 25
    smux = 199
    snmp = 161
    snmp_trap = 162
    snpp = 444
    socks = 1080
    spamd = 783
    ssh = 22
    submission = 587
    submissions = 465
    sunrpc = 111
    supfiledbg = 1127
    supfilesrv = 871
    suucp = 4031
    svn = 3690
    svrloc = 427
    syslog = 514
    syslog_tls = 6514
    sysrqd = 4094
    systat = 11
    tacacs = 49
    talk = 517
    tcpmux = 1
    telnet = 23
    telnets = 992
    tfido = 60177
    tftp = 69
    time = 37
    tinc = 655
    tproxy = 8081
    uucp = 540
    venus = 2430
    venus_se = 2431
    webmin = 10000
    who = 513
    whois = 43
    wnn6 = 22273
    x11 = 6000
    x11_1 = 6001
    x11_2 = 6002
    x11_3 = 6003
    x11_4 = 6004
    x11_5 = 6005
    x11_6 = 6006
    x11_7 = 6007
    xdmcp = 177
    xinetd = 9098
    xmms2 = 9667
    xmpp_client = 5222
    xmpp_server = 5269
    xtel = 1313
    xtelw = 1314
    z3950 = 210
    zabbix_agent = 10050
    zabbix_trapper = 10051
    zebra = 2601
    zebrasrv = 2600
    zephyr_clt = 2103
    zephyr_hm = 2104
    zephyr_srv = 2102
    zip = 6
    zope = 9673
    zope_ftp = 8021
    zserv = 346
